@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.60';
+our $VERSION = '0.70';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -17,14 +17,16 @@ my $max_matrix_value;
 my $need_transpose;
 my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
 my ( $inicial_price, $iter_count_global, $iter_count_local, $epsilon_scaling );
-my ( %index_correlation, %assignned_object, %assignned_person, %price_object, %objects_desired_by_this, %filtered_list, %locked_person );
+my ( $target, $output, $locked_list );
+my ( %index_correlation, %assignned_object, %assignned_person, %price_object );
+my ( %objects_desired_by_this,  );
 
 sub auction { #						=> default values
 	my %args = ( matrix_ref				=> undef,     # reference to array: matrix N x M			                                     
 				maximize_total_benefit  => 0,         # 0: minimize_total_benefit ; 1: maximize_total_benefit
 				inicial_stepsize        => undef,     # auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M)				
 				inicial_price           => 0,
-				verbose                 => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 5, 8, 10: debug information.
+				verbose                 => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 5, 8, 9, 10: debug information.
 				@_,                                   # argument pair list goes here
 				);
        
@@ -32,11 +34,12 @@ sub auction { #						=> default values
 	$iter_count_global = 0;
 	$epsilon_scaling = 0;
 	$need_transpose = 0;
+	$locked_list = 0;
 	%index_correlation = ();
+	%assignned_object = ();
+	%assignned_person = ();
 	%price_object = ();
 	%objects_desired_by_this = ();
-	%filtered_list = ();
-	%locked_person = ();
 	
 	my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM) = $min_size x $max_size
    
@@ -46,6 +49,18 @@ sub auction { #						=> default values
 	$min_size = $array1_size < $array2_size ? $array1_size : $array2_size ; # square matrix --> $min_size = $max_size and $array1_size = $array2_size
 	$max_size = $array1_size < $array2_size ? $array2_size : $array1_size ;
 	$original_max_size = $max_size;
+
+	$target = 'auction-' . $min_size . 'x' . $max_size . '-output.txt' ;
+	
+	if ( $args{verbose} >= 8 ){ # print the verbose messages to $output file.
+		print "\n verbose = $args{verbose} ---> ";
+		if ( open ( $output, '>', $target ) ) {
+			print " *** Open <$target> for writing. *** \n";
+		} else {
+			$args{verbose} = 7;
+			warn " *** Could not open <$target> for writing: $! *** \n";			
+		}
+	}	
    
 	$maximize_total_benefit = $args{maximize_total_benefit};
    
@@ -82,10 +97,10 @@ sub auction { #						=> default values
 	
 	$inicial_price    = $args{inicial_price};
 	$price_object{$_} = $inicial_price for ( 0 .. $max_size - 1 );
-	#$price_object{$_} = sprintf( "%.0f", rand($min_size) ) for ( 0 .. $max_size - 1 ); # random values for initial prices
+	#$price_object{$_} = sprintf( "%.0f", rand($max_matrix_value) ) for ( 0 .. $max_size - 1 ); # random values for initial prices	
    
 	# inicial epsilon value
-	my $epsilon = ($max_matrix_value/2) * exp ( -1 * $max_size/$min_size );  # exp (1) = e = 2.71828182845905
+	my $epsilon = int (($max_matrix_value/2) * exp ( -1 * $max_size/$min_size ));  # exp (1) = e = 2.71828182845905
 	   $epsilon = $args{inicial_stepsize} if ( defined $args{inicial_stepsize} );
 	   $epsilon = 1/(1+$min_size) if ($epsilon < 1/$min_size);
     
@@ -115,17 +130,17 @@ sub auction { #						=> default values
 				foreach my $per ( sort { $a <=> $b } keys %assignned_person){ push @assignment, $assignned_person{$per}; }
 				foreach my $obj ( sort { $a <=> $b } keys %price_object    ){ push @prices, $price_object{$obj}; }
 				my $assig_count = scalar @assignment;
-				print " *** \$iter_count_global = $iter_count_global ; \$assig_count = $assig_count ; \$epsilon = $epsilon ; \@assignment = (@assignment) ; \@prices = (@prices) \n\n";
+				print $output " *** \$iter_count_global = $iter_count_global ; \$assig_count = $assig_count ; \$epsilon = $epsilon ; \@assignment = (@assignment) ; \@prices = (@prices) \n\n";
 			
 				for my $i ( -1 .. $#matrix ) {
-					if ($i >= 0){ printf " %2s  [", $i; } else{ printf "object"; }
+					if ($i >= 0){ printf $output " %2s  [", $i; } else{ printf $output "object"; }
 					for my $j ( 0 .. $#{$matrix[$i]} ) {
-						if ($i >= 0){ printf(" %${matrix_spaces}.${decimals}f", $matrix[$i]->[$j]); } else{ printf(" %${matrix_spaces}.0f", $j); }
-						if ( defined $assignned_person{$i} and $j == $assignned_person{$i} ){ print "**"; } else{ print "  "; }
+						if ($i >= 0){ printf $output (" %${matrix_spaces}.${decimals}f", $matrix[$i]->[$j]); } else{ printf $output (" %${matrix_spaces}.0f", $j); }
+						if ( defined $assignned_person{$i} and $j == $assignned_person{$i} ){ print $output "**"; } else{ print $output "  "; }
 					}
-					if ($i >= 0){ print "]\n"; } else{ print "\n\n"; }
+					if ($i >= 0){ print $output "]\n"; } else{ print $output "\n\n"; }
 				}		
-			}		 
+			}			
 		}
 		
 		$epsilon = $epsilon / 4 ; # (1/2): smooth convergence	  
@@ -169,7 +184,7 @@ sub auction { #						=> default values
 	}}   
    
 	if ( $args{verbose} >= 8 ){
-		printf "\n\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
+		printf $output "\n\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
 	}   
 	print_screen_messages( \@matrix, \@matrix_index, \@matrix_input, \@output_index, $optimal_benefit, $args{verbose}, $epsilon ) ;
        
@@ -218,7 +233,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
       for my $i ( 0 .. $#{$matrix_ref} ) {
          print " [";
          for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
-            printf(" %${matrix_spaces}.${decimals}f", $matrix_ref->[$i]->[$j] );
+            printf (" %${matrix_spaces}.${decimals}f", $matrix_ref->[$i]->[$j] );
 			if ( defined $intersection_columns{$j} and $intersection_columns{$j} == $min_size ){ print "**"; } else{ print "  "; }
          }
          print "]\n";
@@ -250,7 +265,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
 		for my $i ( 0 .. $#{$matrix_ref} ) {
 			print " [";
 			for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
-				printf(" %${matrix_spaces}.${decimals}f  ", $matrix_ref->[$i]->[$j] );
+				printf (" %${matrix_spaces}.${decimals}f  ", $matrix_ref->[$i]->[$j] );
 			}
 			print "]\n";
 		}
@@ -418,15 +433,15 @@ sub auctionRound {
 	my ( %info, %seen_bid );
 
 	if ( $verbose >= 8 ){
-		print "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
+		print $output "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
-			printf " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
+			printf $output " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
 		}
 		foreach my $object ( sort { $a <=> $b } keys %price_object ){
-			printf " \$price_object{%3s} = $price_object{$object} \n", $object;
+			printf $output " \$price_object{%3s} = $price_object{$object} \n", $object;
 		}
-		print "\n";
+		print $output "\n";
 	}
    
 	my $seen_ghost;
@@ -454,7 +469,7 @@ sub auctionRound {
 			my %current_value;
 			my @updated_price;
 			
-			$seen_ghost++ if ( not defined $matrix[$person] and $min_size < $max_size );
+			$seen_ghost++ if ( not defined $matrix[$person] and $min_size < $max_size );						
 
 			# The @objects_with_greater_benefits list are updated dynamically by $objects_desired_by_this{$person} by considering the current price (or current value) of objects.		
 			# For each person, the Current Value $current_value{$object} of the most desired objects contained in the @objects_with_greater_benefits list tends to converge to a specific value.
@@ -490,16 +505,20 @@ sub auctionRound {
 				{
 					$Opt03ValForPersonI = $current_value{$object};
 					$Opt03ObjForPersonI = $object;
-				}								
-
+				}
+				
 				if ( $verbose >= 8 ){
-					printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <old list> \n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object} ;
+					printf $output " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <old list> Max01_CurVal = %12.5f (%3s), Max02_CurVal = %12.5f (%3s), Max03_CurVal = %12.5f (%3s)\n", 
+					$person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, 
+					$Opt01ValForPersonI, defined $Opt01ObjForPersonI ? $Opt01ObjForPersonI : '', 
+					$Opt02ValForPersonI, defined $Opt02ObjForPersonI ? $Opt02ObjForPersonI : '', 
+					$Opt03ValForPersonI, defined $Opt03ObjForPersonI ? $Opt03ObjForPersonI : '';
 				}
 			}
-
-			if ( not @updated_price and not defined $locked_person{$person} )  # if all prices are outdated
+			
+			if ( not @updated_price and not $locked_list ) # if all prices are outdated
 			{				
-				if ( $epsilon_scaling <= 2 )
+				if ( $epsilon_scaling <= 2 or $epsilon > 64 ) # 1000/(4**2) ~ 64 ; 1000000/(4**7) ~ 64 ; $max_matrix_value/(4**x) ~ 4**3 --> $max_matrix_value/(4**3) ~ (4**x) --> x ~ [ln ($max_matrix_value/(4**3))]/[ln(4)]
 				{
 					for my $object ( 0 .. $max_size - 1 ) # generate new list
 					{
@@ -508,7 +527,7 @@ sub auctionRound {
 						my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
 						$current_value{$object} = $matrix_value - $price_object{$object};
 				
-						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best 3 objects in the complementary subset
+						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best 3 objects in the complementary subset <new list>
 						{
 							$Opt03ValForPersonI_new_list = $Opt02ValForPersonI_new_list;
 							$Opt03ObjForPersonI_new_list = $Opt02ObjForPersonI_new_list;
@@ -531,11 +550,15 @@ sub auctionRound {
 						}
 
 						if ( $verbose >= 8 ){
-							printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f ; \$Opt01ValForPersonI_new_list = %12.5f\n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, $Opt01ValForPersonI_new_list;
+							printf $output " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <new list> Max01_CurVal = %12.5f (%3s), Max02_CurVal = %12.5f (%3s), Max03_CurVal = %12.5f (%3s)\n", 
+							$person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, 
+							$Opt01ValForPersonI_new_list, defined $Opt01ObjForPersonI_new_list ? $Opt01ObjForPersonI_new_list : '', 
+							$Opt02ValForPersonI_new_list, defined $Opt02ObjForPersonI_new_list ? $Opt02ObjForPersonI_new_list : '', 
+							$Opt03ValForPersonI_new_list, defined $Opt03ObjForPersonI_new_list ? $Opt03ObjForPersonI_new_list : '';
 						}				
 					}
 				} 
-				elsif ( $epsilon_scaling >= 3 and $epsilon_scaling <= 6 )
+				elsif ( ($epsilon_scaling >= 3 and $epsilon_scaling <= 6) or $epsilon > 1 )
 				{
 					for my $object ( 0 .. $max_size - 1 ) # generate new list
 					{
@@ -544,7 +567,7 @@ sub auctionRound {
 						my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
 						$current_value{$object} = $matrix_value - $price_object{$object};
 				
-						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best 2 objects in the complementary subset
+						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best 2 objects in the complementary subset <new list>
 						{
 							$Opt02ValForPersonI_new_list = $Opt01ValForPersonI_new_list;
 							$Opt02ObjForPersonI_new_list = $Opt01ObjForPersonI_new_list;
@@ -558,11 +581,15 @@ sub auctionRound {
 						}
 
 						if ( $verbose >= 8 ){
-							printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f ; \$Opt01ValForPersonI_new_list = %12.5f\n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, $Opt01ValForPersonI_new_list;
+							printf $output " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <new list> Max01_CurVal = %12.5f (%3s), Max02_CurVal = %12.5f (%3s), Max03_CurVal = %12.5f (%3s)\n", 
+							$person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, 
+							$Opt01ValForPersonI_new_list, defined $Opt01ObjForPersonI_new_list ? $Opt01ObjForPersonI_new_list : '', 
+							$Opt02ValForPersonI_new_list, defined $Opt02ObjForPersonI_new_list ? $Opt02ObjForPersonI_new_list : '', 
+							$Opt03ValForPersonI_new_list, defined $Opt03ObjForPersonI_new_list ? $Opt03ObjForPersonI_new_list : '';
 						}				
 					}
 				}
-				elsif ( $epsilon_scaling >= 7 ) # in most cases, need only one object
+				elsif ( $epsilon_scaling >= 7 ) # in most cases, only one object is found in the complementary subset <new list> that has high value.
 				{
 					for my $object ( 0 .. $max_size - 1 ) # generate new list
 					{
@@ -571,14 +598,18 @@ sub auctionRound {
 						my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
 						$current_value{$object} = $matrix_value - $price_object{$object};
 				
-						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best object in the complementary subset
+						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best object in the complementary subset <new list>
 						{
 							$Opt01ValForPersonI_new_list = $current_value{$object};
 							$Opt01ObjForPersonI_new_list = $object;
 						}
 
 						if ( $verbose >= 8 ){
-							printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f ; \$Opt01ValForPersonI_new_list = %12.5f\n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, $Opt01ValForPersonI_new_list;
+							printf $output " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <new list> Max01_CurVal = %12.5f (%3s), Max02_CurVal = %12.5f (%3s), Max03_CurVal = %12.5f (%3s)\n", 
+							$person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, 
+							$Opt01ValForPersonI_new_list, defined $Opt01ObjForPersonI_new_list ? $Opt01ObjForPersonI_new_list : '', 
+							$Opt02ValForPersonI_new_list, defined $Opt02ObjForPersonI_new_list ? $Opt02ObjForPersonI_new_list : '', 
+							$Opt03ValForPersonI_new_list, defined $Opt03ObjForPersonI_new_list ? $Opt03ObjForPersonI_new_list : '';
 						}				
 					}
 				}			
@@ -606,7 +637,7 @@ sub auctionRound {
 					$Opt03ValForPersonI = $Opt01ValForPersonI_new_list;
 					$Opt03ObjForPersonI = $Opt01ObjForPersonI_new_list;
 				}
-
+ 
 				if ( $Opt02ValForPersonI_new_list > $Opt02ValForPersonI )
 				{
 					$Opt03ValForPersonI = $Opt02ValForPersonI;
@@ -632,42 +663,36 @@ sub auctionRound {
 			# Stores the bidding info for future use
 			$info{$Opt01ObjForPersonI}{$bid01ForPersonI} = $person if ( not defined $info{$Opt01ObjForPersonI}{$bid01ForPersonI} ); # get only one person with this bid			
 
-			if ( $max_size >= 6 ){
-			
+			if ( $max_size >= 6 )
+			{	
 				$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
 				$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
 
-				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );				
-					
-				if ( not @updated_price ){ # if all prices are outdated
-					
-					$objects_desired_by_this{$person}{$_} = $current_value{$_} for ( @objects_with_greater_benefits ); # update current values for all objects in the old list
-					
-					if ( $filtered_list{$person} )
-					{
-						$Opt01ValForPersonI_new_list = $locked_person{$person} if ( defined $locked_person{$person} );
-						
-						# Lock or unlock the old list after reviewing the quality of the list objects
-						if ( ($Opt03ValForPersonI - $Opt01ValForPersonI_new_list) > 4 * $epsilon )
-						{
-							$Opt01ValForPersonI_new_list -= $epsilon;
-							$locked_person{$person} = $Opt01ValForPersonI_new_list; # Lock the old list
-							printf "<> PersonI = %3s ; dif %.5f ( \$Opt03ValForPersonI %.5f - \$Opt01ValForPersonI_new_list %.5f ) > (4 * \$epsilon) %.5f ; (Max_CurVal %.5f) *** locked person *** \n", $person, $Opt03ValForPersonI - $Opt01ValForPersonI_new_list, $Opt03ValForPersonI, $Opt01ValForPersonI_new_list, 4 * $epsilon, $locked_person{$person} if ( $verbose >= 8 );
-						} elsif ( defined $locked_person{$person} ) {							
-							printf "<> PersonI = %3s ; dif %.5f ( \$Opt03ValForPersonI %.5f - \$Opt01ValForPersonI_new_list %.5f ) <= (4 * \$epsilon) %.5f ; (Max_CurVal %.5f) *** unlocked person *** \n", $person, $Opt03ValForPersonI - $Opt01ValForPersonI_new_list, $Opt03ValForPersonI, $Opt01ValForPersonI_new_list, 4 * $epsilon, $locked_person{$person} if ( $verbose >= 8 );
-							delete $filtered_list{$person}; # filter again
-							delete $locked_person{$person}; # Unlock the old list
-						}
+				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );
+
+				if ( $epsilon_scaling >= 6 and $epsilon_scaling % 3 == 0 ) # filter the old list
+				{					
+					for my $object ( @objects_with_greater_benefits ){
+						next if ( $current_value{$object} >= $Opt03ValForPersonI );
+						next unless ( ($Opt03ValForPersonI - $current_value{$object}) > 2 * $min_size * $epsilon );
+						delete $objects_desired_by_this{$person}{$object}; # delete objects of little benefit from the old list
+						printf $output "<> PersonI = %3s ; *** delete the object %3s from the old list *** \n", $person, $object if ( $verbose >= 8 );
 					}
-					elsif ( $epsilon_scaling >= 6 ) # filter the old list
+					$locked_list++;
+				}
+					
+				if ( not @updated_price ) # if all prices are outdated
+				{			
+					for my $object ( @objects_with_greater_benefits ){
+						next unless ( defined $objects_desired_by_this{$person}{$object} );
+						$objects_desired_by_this{$person}{$object} = $current_value{$object}; # update current values for all objects in the old list	
+					}
+					
+					if ( $locked_list and defined $Opt01ObjForPersonI_new_list and $Opt03ValForPersonI == $Opt01ValForPersonI_new_list and defined $objects_desired_by_this{$person}{$Opt03ObjForPersonI} )
 					{
-						for my $object ( @objects_with_greater_benefits ){
-							$filtered_list{$person} = 1;
-							next unless ( ($Opt01ValForPersonI - $current_value{$object}) > 1 * $min_size * $epsilon );
-							delete $objects_desired_by_this{$person}{$object}; # delete objects of little benefit from the old list 							
-							printf "<> PersonI = %3s ; *** delete the object %3s from the old list *** \n", $person, $object if ( $verbose >= 8 );
-						}
-					}					
+						$objects_desired_by_this{$person}{$Opt01ObjForPersonI_new_list} = $current_value{$Opt01ObjForPersonI_new_list}; # add object to list
+						printf $output "<> PersonI = %3s ; *** add object %3s to the old list *** \n", $person, $Opt01ObjForPersonI_new_list if ( $verbose >= 8 );
+					}										
 					
 					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # information about the most desired objects
 					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $current_value{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
@@ -679,11 +704,12 @@ sub auctionRound {
 				my @old_list = sort { $a <=> $b } @objects_with_greater_benefits;
 				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
 				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
+				my $msg = $locked_list ? 'locked list' : '';
 
-				printf "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
-				printf "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects)\n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
-				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
-				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
+				printf $output "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
+				printf $output "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) $msg \n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
+				printf $output "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
+				printf $output "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
 		}
 	}
@@ -697,7 +723,7 @@ sub auctionRound {
 		if ( defined $other_person ) {
 		  
 			if ( $verbose >= 8 ){
-				print " ***--> PersonI $person was assigned objectJ $object. Before that, remove the objectJ $object from personI $other_person  --> delete \$assignned_person{$other_person} \n";
+				print $output " ***--> PersonI $person was assigned objectJ $object. Before that, remove the objectJ $object from personI $other_person  --> delete \$assignned_person{$other_person} \n";
 			}
 		  
 			# The other person that was assigned to objectJ at the beginning of the iteration (if any) 
@@ -715,22 +741,22 @@ sub auctionRound {
 		$price_object{$object} += $bid;
 	   
 		if ( $verbose >= 8 ){
-			printf " --> Assigning to personI = %2s the objectJ = %2s with highestBidForJ = %10.5f and update the price vector ; \$assignned_person{%2s} = %2s ; \$price_object{%2s} = %.5f \n", $person, $object, $bid, $person, $assignned_person{$person}, $object, $price_object{$object};	
+			printf $output " --> Assigning to personI = %3s the objectJ = %3s with highestBidForJ = %10.5f and update the price vector ; \$assignned_person{%3s} = %3s ; \$price_object{%3s} = %.5f \n", $person, $object, $bid, $person, $assignned_person{$person}, $object, $price_object{$object};	
 		}
 	   
 		last;  # next object, choose only the highest bid for each object
 	}}
 	
 	if ( $verbose >= 9 ){
-		print "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
+		print $output "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
-			printf " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
+			printf $output " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
 		}
 		foreach my $object ( sort { $a <=> $b } keys %price_object ){
-			printf " \$price_object{%3s} = $price_object{$object} \n", $object;
+			printf $output " \$price_object{%3s} = $price_object{$object} \n", $object;
 		}
-		print "\n";
+		print $output "\n";
 	}
 
 }
@@ -948,7 +974,7 @@ __END__
  maximize_total_benefit => 0,    0: minimize the total benefit ; 1: maximize the total benefit.
  inicial_stepsize       => 1,    auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M).
  inicial_price          => 0,			
- verbose                => 3,    print messages on the screen, level of verbosity, 0: quiet; 1, 2, 3, 4, 5, 8, 10: debug information.
+ verbose                => 3,    print messages on the screen, level of verbosity, 0: quiet; 1, 2, 3, 4, 5, 8, 9, 10: debug information.
 
 =head1 EXPORT
 
@@ -987,7 +1013,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	April 13, 2018
+	April 15, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
